@@ -40,8 +40,8 @@ from xasyActions import *
 import string
 
 try:
-  import ImageTk
-  import Image
+  from PIL import ImageTk
+  from PIL import Image
   PILAvailable = True
 except:
   PILAvailable = False
@@ -57,7 +57,7 @@ class xasyMainWin:
     self.createWidgets()
     self.resetGUI()
     if sys.platform[:3] == "win":
-      site="http://effbot.org/downloads/PIL-1.1.7.win32-py2.6.exe"
+      site="http://effbot.org/downloads/PIL-1.1.7.win32-py2.7.exe"
     else:
       site="http://effbot.org/downloads/Imaging-1.1.7.tar.gz" 
     if not PILAvailable:
@@ -781,7 +781,7 @@ class xasyMainWin:
 
   def helpAsyDocCmd(self):
     #print "Open documentation about Asymptote"
-    asyExecute("\nhelp;\n")
+    asyExecute("help;\n")
 
   def helpAboutCmd(self):
     tkMessageBox.showinfo("About xasy","A graphical interface for Asymptote "+xasyVersion)
